from uvtor.conf import settings
from uvtor.core.cache import cache
from uvtor.db.models import manager
from uvtor.db.models.user import User
import tornado.web
import functools
import pickle


class BaseHandler(tornado.web.RequestHandler):

    def check_xsrf_cookie(self):

        if settings.DEBUG:
            return True
        else:
            super(BaseHandler, self).check_xsrf_cookie()

    def _args(self, *keys):
        return map(
            functools.partial(
                self.get_argument, default=None),
            keys)

    def _secure_cookies(self, *keys):
        return map(
            self.get_secure_cookie,
            keys)

    async def get_current_user(self):
        print(await manager.get(User, id=1))
        secure_token = self.get_secure_cookie('_secure')
        if not secure_token:
            return None
        if isinstance(secure_token, bytes):
            secure_token = secure_token.decode()
        _ = await cache.get(str.format('{}:{}',
                                       settings.SESSION_PREFIX,
                                       secure_token))
        if not _:
            return None
        _user = pickle.loads(
                _
            )
        if not _user:
            return None
        return await manager.get(User, id=_user.get('user_id'))



from argparse import ArgumentParser
from io import TextIOBase
import sys
import os
import uvtor


class CommandError(Exception):

    pass


class CommandParser(ArgumentParser):

    def __init__(self, cmd, **kwargs):
        self.cmd = cmd
        super().__init__(**kwargs)

    def parse_args(self, args=None, namespace=None):
        if (hasattr(self.cmd, 'missing_args_message') and
                not (args or any(not arg.startswith('-') for arg in args))):
            self.error(self.cmd.missing_args_message)
        return super().parse_args(args, namespace)

    def error(self, message):
        if self.cmd._called_from_command_line:
            super().error(message)
        else:
            raise CommandError("Error: %s" % message)


class OutputWrapper(TextIOBase):
    """
    Wrapper around stdout/stderr
    """
    @property
    def style_func(self):
        return self._style_func

    @style_func.setter
    def style_func(self, style_func):
        if style_func and self.isatty():
            self._style_func = style_func
        else:
            self._style_func = lambda x: x

    def __init__(self, out, style_func=None, ending='\n'):
        self._out = out
        self.style_func = None
        self.ending = ending

    def __getattr__(self, name):
        return getattr(self._out, name)

    def isatty(self):
        return hasattr(self._out, 'isatty') and self._out.isatty()

    def write(self, msg, style_func=None, ending=None):
        ending = self.ending if ending is None else ending
        if ending and not msg.endswith(ending):
            msg += ending
        style_func = style_func or self.style_func
        self._out.write(style_func(msg))


class BaseCommand:

    help = ''

    # Configuration shortcuts that alter various logic.
    _called_from_command_line = False
    # Whether to wrap the output in a "BEGIN; COMMIT;"
    output_transaction = False
    leave_locale_alone = False
    requires_migrations_checks = False
    requires_system_checks = True
    # Arguments, common to all commands, which aren't defined by the argument
    # parser.
    base_stealth_options = ('skip_checks', 'stderr', 'stdout')
    # Command-specific options not defined by the argument parser.
    stealth_options = ()

    def __init__(self, stdout=None, stderr=None):
        self.stdout = OutputWrapper(stdout or sys.stdout)
        self.stderr = OutputWrapper(stderr or sys.stderr)

    def get_version(self):

        return uvtor.VERSION

    def create_parser(self, prog_name, subcommand):
        """
        Create and return the ``ArgumentParser`` which will be used to
        parse the arguments to this command.
        """
        parser = CommandParser(
            self, prog="%s %s" % (os.path.basename(prog_name), subcommand),
            description=self.help or None,
        )
        parser.add_argument(
            '--version', action='version', version=self.get_version())

        self.add_arguments(parser)
        return parser

    def add_arguments(self, parser):
        """
        Entry point for subclassed commands to add custom arguments.
        """
        pass

    def print_help(self, prog_name, subcommand):
        """
        Print the help message for this command, derived from
        ``self.usage()``.
        """
        parser = self.create_parser(prog_name, subcommand)
        parser.print_help()

    def run_from_argv(self, argv):
        """
        Set up any environment changes requested (e.g., Python path
        and Django settings), then run this command. If the
        command raises a ``CommandError``, intercept it and print it sensibly
        to stderr. If the ``--traceback`` option is present or the raised
        ``Exception`` is not ``CommandError``, raise it.
        """
        self._called_from_command_line = True
        parser = self.create_parser(argv[0], argv[1])
        options = parser.parse_args(argv[2:])
        cmd_options = vars(options)
        # Move positional args out of options to mimic legacy optparse
        args = cmd_options.pop('args', ())
        self.check()
        try:
            self.execute(*args, **cmd_options)
        except Exception as e:

            self.stderr.write('%s: %s' % (e.__class__.__name__, e))

            sys.exit(1)

    def check(self):
        pass

    def execute(self, *args, **options):
        """
        Try to execute this command, performing system checks if needed (as
        controlled by the ``requires_system_checks`` attribute, except if
        force-skipped).
        """
        raise NotImplementedError



from uvtor.core import exceptions


async def _check_login(client):
    _user = await client.get_current_user()
    if not _user:
        raise exceptions.LoginRequired
    return _user


def ensure_xsrf(func):

    async def decorator(client, *args, **kwargs):
        client.set_cookie('_xsrf', client.xsrf_token)
        return await func(client, *args, **kwargs)

    return decorator


def login_required(func):

    async def decorator(client, *args, **kwargs):
        await _check_login(client)
        return await func(client, *args, **kwargs)

    return decorator



from uvtor.core.management.base import BaseCommand, CommandError
import os
import shutil
import re


class Command(BaseCommand):

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of app',
        )

    def _is_valid_name(self, name):
        if not re.match(r'^[a-zA-Z]+[\w_]+$', name):
            raise CommandError('Invalid project name')

    def execute(self, **options):
        _appname = options.get('name')

        shutil.rmtree(os.path.join('apps', _appname))
        shutil.rmtree(os.path.join('tests', _appname))

        print(f'Successfully remove app {_appname}')



from uvtor.core.management.base import BaseCommand, CommandError
from uvtor.conf import settings
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
import psycopg2
import os


class Command(BaseCommand):

    def check(self):
        if not os.path.exists('.uvtor'):
            raise CommandError('Invalid uvtor project')

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument('-e', '--env', type=str,
                            help='Env.', default='TEST')
        parser.add_argument(
            '--defaultdb', '-d',
            help='Default db connection session.',
            metavar=('DB_NAME'))

    def _get_conn(self, default_db, env='TEST'):
        conf = getattr(settings, 'ENVS')[env]
        con = psycopg2.connect(database=default_db, user=conf.get(
            'user'), host=conf.get('host'), password=conf.get('password'))
        con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
        return con

    def _drop_db(self, env, name, default_db):
        conf = getattr(settings, 'ENVS')[env]
        _name = conf.get('database')
        con = self._get_conn(default_db, env)
        cur = con.cursor()
        cur.execute(f'DROP DATABASE {_name} ;')

        print(f'Drop database {_name} successfully')

    def execute(self, **options):
        _env = options.get('env')
        _name = options.get('name')
        _db = options.get('defaultdb')

        print('Using env %s' % _env)
        self._drop_db(_env, _name, _db)



from tornado.test.util import unittest
from uvtor.core.management.base import BaseCommand
import os
import sys
import pkgutil


class Command(BaseCommand):

    TEST_MODULES = [
    ]

    def all(self):
        return unittest.defaultTestLoader.loadTestsFromNames(
            self.TEST_MODULES)

    def _get_tests(self, *modules):
        if len(modules) > 0:
            for _module in set(modules):
                print(_module)
                module_ = __import__(f'tests.{_module}')
                for loader, modname, ispkg in pkgutil.walk_packages(
                        module_.__path__, module_.__name__ + '.'):
                    if not ispkg:
                        self.TEST_MODULES.append(modname)
        else:
            _tests = __import__('tests')
            for loader, modname, ispkg in pkgutil.walk_packages(
                    _tests.__path__, _tests.__name__ + '.'):
                if not ispkg:
                    self.TEST_MODULES.append(modname)

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '--modules', metavar='name',
            nargs='+',
            help='Name of app',
            default=[]
        )

    def execute(self, **options):
        sys.path.insert(0, os.getcwd())
        self._get_tests(*options.get('modules'))
        unittest.TextTestRunner().run(self.all())




CODE_0_SUCCESS = 0
CODE_1_ERROR = 1
CODE_2_LOGIN_REQUIRED = 2
CODE_3_ERROR_SECURE_TOKEN = 3
CODE_4_ERROR_USERNAME_OR_PASSWORD = 4
CODE_5_INVALID_PASSWORD = 5
CODE_6_USER_CREATION_FAILED = 6
CODE_7_SEND_SMS_FAILED = 7
CODE_8_CAPTCHA_VALIDATION_ERROR = 8
CODE_9_INVALID_PHONE_NUMBER = 9
CODE_10_INVALID_EMAIL_ADDRESS = 10
CODE_11_INVALID_USERNAME = 11
CODE_12_INVALID_CHINESE_NAME = 12
CODE_13_VCODE_VALIDATION_ERROR = 13
CODE_14_PHONE_ALREADY_EXISTS_ERROR = 14
CODE_15_USERNAME_ALREADY_EXISTS_ERROR = 15
CODE_16_EMAIL_ALREADY_EXISTS_ERROR = 16
CODE_19_MISSING_REQUIRED_PARAMS = 19
CODE_20_PERMISSION_DENIED = 20
CODE_21_INVALID_PARAMETER = 21
CODE_23_OBJECT_ALREADY_EXISTS = 23
CODE_24_USER_IS_DISABLED = 24



import functools


def singleton(cls):
    ''' Use class as singleton. '''

    cls.__new_original__ = cls.__new__

    @functools.wraps(cls.__new__)
    def singleton_new(cls, *args, **kw):
        it = cls.__dict__.get('__it__')
        if it is not None:
            return it

        cls.__it__ = it = cls.__new_original__(cls, *args, **kw)
        it.__init_original__(*args, **kw)
        return it

    cls.__new__ = singleton_new
    cls.__init_original__ = cls.__init__
    cls.__init__ = object.__init__

    return cls



"""
    The package description.

"""
import datetime
import logging

import peewee as pw

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.StreamHandler())
LOGGER.setLevel(logging.INFO)


class MigrateHistory(pw.Model):

    """Presents the migrations in database."""

    name = pw.CharField()
    migrated_at = pw.DateTimeField(default=datetime.datetime.utcnow)

    def __unicode__(self):
        """String representation."""
        return self.name


from .router import Migrator, Router  # noqa



from tornado.platform.asyncio import AsyncIOMainLoop
from tornado.log import enable_pretty_logging
from uvtor.core.management.base import BaseCommand, CommandError
from uvtor.core import make_app
import asyncio
import uvloop
import os
import logging


asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

AsyncIOMainLoop().install()


class Command(BaseCommand):

    def check(self):
        if not os.path.exists('.uvtor'):
            raise CommandError('Invalid uvtor project')

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of app',
        )
        parser.add_argument(
            '--port', '-P', metavar='port',
            default=8000,
            help='Listen port',
        )

        parser.add_argument(
            '--address', '-H', metavar='address',
            default='127.0.0.1',
            help='Listen address',
        )
        parser.add_argument(
            '--autoreload',
            help='Enable autoreload',
            action='store_true'
        )

    def execute(self, **options):
        _appname = options.get('name')
        _address = options.get('address')
        _port = options.get('port')
        _autoreload = options.get('autoreload')
        _app = make_app(_appname, _autoreload)
        _app.listen(_port, address=_address)
        enable_pretty_logging()
        logging.info(f'Server started, listen on {_address}:{_port}')
        try:
            asyncio.get_event_loop().run_forever()
        except KeyboardInterrupt as e:
            asyncio.get_event_loop().close()



from uvtor.core.management.base import BaseCommand
from uvtor.conf import settings
import os
import datetime


class Command(BaseCommand):
    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument('--env', '-e', type=str,
                            help='Env.', default='TEST')

    def _init_file(self, filename):
        with open(filename, 'w') as file:
            file.write("""expressions = [

    ]
    """)

    def _create_migration_file(self):
        directory = os.path.join(settings.ROOT_DIR, 'models', 'migrations')

        _index = max([int(_file.split('_')[0]) for _file in os.listdir(
            directory) if _file.split('_')[0].isdigit()]) + 1
        _file = os.path.join(directory, '%04d_auto_%s.py' % (
            _index, datetime.datetime.strftime(
                datetime.datetime.now(), '%Y%m%d_%H%M')))
        os.mknod(_file)
        self._init_file(_file)
        return True

    def execute(self, **options):
        _env = options.get('env')
        print('Using env %s' % _env)
        self._create_migration_file()
        print('New migration file is successfully created')



from uvtor.core import response as _resp
from uvtor.core.exceptions import APIException
import traceback


def standard_api(func):

    async def decorator(client, *args, **kwargs):
        try:
            result = await func(client, *args, **kwargs)
            if isinstance(result, dict) or not result:
                response = {'code': _resp.CODE_0_SUCCESS}
                response.update(result or {})
                client.write(response)
            else:
                client.write(result)
        except (APIException, Exception) as e:
            # print(traceback.format_exc())
            if not hasattr(e, 'code'):
                print(traceback.format_exc())
                e.code = 1
            client.write({'code': e.code, 'error_msg': str(e)})

    return decorator



from playhouse import postgres_ext
from uvtor.core import exceptions
from uvtor.core.cache import cache
from uvtor.conf import settings
from utilib.utils import security
from utilib.utils import timezone
from uvtor.db.models import manager
from uvtor.db.models import BaseModel
from datetime import datetime
import peewee
import base64
import uuid
import pickle


class User(BaseModel):
    date_joined = postgres_ext.DateTimeField(default=datetime.now())
    email = postgres_ext.CharField(null=True, unique=True)
    first_name = postgres_ext.CharField(null=True)
    is_active = postgres_ext.BooleanField(default=True)
    is_superuser = postgres_ext.BooleanField(default=False)
    last_login = postgres_ext.DateTimeField(null=True)
    last_name = postgres_ext.CharField(null=True)
    password = postgres_ext.CharField()
    salt = postgres_ext.CharField()
    phone = postgres_ext.CharField(null=True)
    username = postgres_ext.CharField(unique=True)

    class Meta:
        db_table = '_user'

    def __str__(self):
        return str.format('{username}, {phone}',
                          username=self.username,
                          phone=self.phone)

    def _check_password(self, password):
        return security.get_password(password, self.salt) == self.password

    def check_password(self, password):
        if not self._check_password(password):
            raise exceptions.UsernameOrPasswordError

    async def _set_password(self, password):
        _password, _salt = security.gen_password(password)
        await manager.execute(
            User.update(password=_password,
                        salt=_salt).where(User.id == self.id))

    @staticmethod
    async def create_user(username, password, phone=None, email=None,
                          first_name=None, last_name=None):
        _password, _salt = security.gen_password(password)
        try:
            return await manager.execute(
                User.insert(username=username,
                            password=_password,
                            salt=_salt,
                            phone=phone,
                            email=email,
                            first_name=first_name,
                            last_name=last_name))
        except peewee.IntegrityError:
            raise exceptions.UserCreationError

    @staticmethod
    async def login(_, password, secure_token=None):
        if isinstance(secure_token, bytes):
            secure_token = secure_token.decode()
        try:
            _user = await manager.get(
                User.select().where((User.username == _)
                                    | (User.phone == _)
                                    | (User.email == _)))
            if not _user.is_active:
                raise exceptions.UserIsDisabled
            _user.check_password(password)
        except Exception:
            if secure_token:
                await cache.delete(str.format('{}:{}',
                                              settings.SESSION_PREFIX,
                                              secure_token))
            raise exceptions.UsernameOrPasswordError
        _user.last_login = timezone.now()
        login_info = {}
        login_info['login_time'] = timezone.now()
        login_info['user_id'] = _user.id
        _secure = None
        if await cache.get(str.format('{}:{}',
                                      settings.SESSION_PREFIX,
                                      secure_token)):
            _secure = secure_token
        else:
            _secure = base64.b64encode(
                uuid.uuid4().bytes + uuid.uuid4().bytes).decode()
        await cache.set(str.format('{}:{}',
                                   settings.SESSION_PREFIX,
                                   _secure),
                        pickle.dumps(login_info))
        await manager.update(_user)
        return _secure

    async def logout(secure_token):
        if isinstance(secure_token, bytes):
            secure_token = secure_token.decode()
        await cache.delete(str.format('{}:{}',
                                      settings.SESSION_PREFIX,
                                      secure_token))

    @staticmethod
    async def validate(phone=None,
                       username=None,
                       email=None,
                       raise_exception=False):
        users = await manager.execute(
            User.select().where((User.phone == phone)
                                | (User.username == username)
                                | (User.email == email)))
        if len(users) == 0:
            return True
        _phones = [_user.phone for _user in users]
        _usernames = [_user.username for _user in users]
        _emails = [_user.email for _user in users]

        if raise_exception and phone in _phones and phone:
            raise exceptions.PhoneAlreadyExistsError
        elif raise_exception and username in _usernames and username:
            raise exceptions.UsernameAlreadyExistsError
        elif raise_exception and email in _emails and email:
            raise exceptions.EmailAlreadyExistsError
        return False



from uvtor.core.management.base import BaseCommand, CommandError
from uvtor.db.migrations import Router
from uvtor.conf import settings
from playhouse.pool import PooledPostgresqlExtDatabase
import pkgutil
import os

CURDIR = os.getcwd()
DEFAULT_MODEL_DIR = os.path.join(CURDIR, 'models')


class Command(BaseCommand):
    def check(self):
        if not os.path.exists('.uvtor'):
            raise CommandError('Invalid uvtor project')

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument('-e', '--env', type=str,
                            help='Env.', default='TEST')
        parser.add_argument(
            '-m',
            '--model',
            type=str,
            help='Set path to your models module.',
            default=None)

    def execute(self, *args, **options):
        self.check()
        _env = options.get('env')
        _model = options.get('model')
        conf = getattr(settings, 'ENVS')[_env]
        db = PooledPostgresqlExtDatabase(
            conf.get('database'), user=conf.get('user'),
            host=conf.get('host'),
            password=conf.get('password'),
            register_hstore=False)
        db.connect()
        router = Router(db)
        if not _model:
            _models = pkgutil.importlib.import_module('models')
            for loader, modname, ispkg in pkgutil.iter_modules(
                    _models.__path__, prefix='models.'):
                _m = pkgutil.importlib.import_module(modname)
                for _, _ins in _m.__dict__.items():
                    setattr(_models, _, _ins)

            router.create(auto=_models)
        else:
            _model = f'models.{_model}'
            router.create(auto=_model)
        db.close()



from uvtor.conf import settings
import tornado.web
import pkgutil
import importlib


def make_app(appname, autoreload=True):
    _settings = {'cookie_secret': settings.COOKIE_SECRET, 'xsrf_cookies': True}
    app = tornado.web.Application(
        auto_discover(appname),
        debug=settings.DEBUG,
        autoreload=autoreload,
        **_settings)
    return app


def auto_discover(appname):
    routes = importlib.import_module('apps.{}.routes'.format(appname))
    name = routes.__name__

    _routes = []
    for loader, modname, ispkg in pkgutil.walk_packages(routes.__path__):
        _module = loader.find_module(modname).load_module(modname)
        if not hasattr(_module, 'routes'):
            continue
        for _, _handler in _module.routes:
            if settings.DEBUG:
                if _ == r'^$':
                    _ = '^%s' % ('%s.%s' % (name, modname)).replace(
                        '.', '/').replace(
                        f'apps/{appname}/routes',
                        f'/api/{appname}')
                else:
                    _ = _.replace(
                        '^/', '^%s/' %
                        ('%s.%s' %
                         (name, modname)).replace(
                            '.', '/').replace(
                            f'apps/{appname}/routes',
                            f'/api/{appname}'))
            else:
                if _ == r'^$':
                    _ = '^%s' % ('%s.%s' % (name, modname)).replace(
                            '.', '/').replace(
                            'apps/{}/routes'.format(appname), '')
                else:
                    _ = _.replace(
                        '^/',
                        '^%s/' % ('%s.%s' % (name, modname)).replace(
                            '.', '/').replace(
                            'apps/{}/routes'.format(appname), ''))
            _routes.append((_, _handler))
    return _routes



from collections import defaultdict
from contextlib import suppress
from importlib import import_module
from uvtor.core.management.base import (
    BaseCommand, CommandError, CommandParser
)
import functools
import os
import pkgutil
import sys
import uvtor


def find_commands(management_dir):
    """
    Given a path to a management directory, return a list of all the command
    names that are available.
    """
    command_dir = os.path.join(management_dir, 'commands')
    return [name for _, name, is_pkg in pkgutil.iter_modules([command_dir])
            if not is_pkg and not name.startswith('_')]


def load_command_class(app_name, name):
    """
    Given a command name and an application name, return the Command
    class instance. Allow all errors raised by the import process
    (ImportError, AttributeError) to propagate.
    """
    module = import_module('%s.management.commands.%s' % (app_name, name))
    return module.Command()


@functools.lru_cache(maxsize=None)
def get_commands():

    commands = {name: 'uvtor.core' for name in find_commands(
        __path__[0])}
    return commands


class ManagementUtility:

    def __init__(self, argv=None):
        self.argv = argv or sys.argv[:]
        self.prog_name = os.path.basename(self.argv[0])
        self.settings_exception = None

    def main_help_text(self, commands_only=False):
        """Return the script's main help text, as a string."""
        if commands_only:
            usage = sorted(get_commands())
        else:
            usage = [
                "",
                (
                    "Type '%s help <subcommand>' "
                    "for help on a specific subcommand." % self.prog_name),
                "",
                "Available subcommands:",
            ]
            commands_dict = defaultdict(lambda: [])
            for name, app in get_commands().items():
                if app == 'uvtor.core':
                    app = 'uvtor'
                else:
                    app = app.rpartition('.')[-1]
                commands_dict[app].append(name)
            for app in sorted(commands_dict):
                usage.append("")
                usage.append("[%s]" % app)
                for name in sorted(commands_dict[app]):
                    usage.append("    %s" % name)

        return '\n'.join(usage)

    def fetch_command(self, subcommand):

        commands = get_commands()
        try:
            app_name = commands[subcommand]
        except KeyError:

            sys.stderr.write(
                "Unknown command: %r\nType '%s help' for usage.\n"
                % (subcommand, self.prog_name)
            )
            sys.exit(1)
        if isinstance(app_name, BaseCommand):
            # If the command is already loaded, use it directly.
            klass = app_name
        else:
            klass = load_command_class(app_name, subcommand)
        return klass

    def execute(self):
        """
        Given the command-line arguments, figure out which subcommand is being
        run, create a parser appropriate to that command, and run it.
        """
        try:
            subcommand = self.argv[1]
        except IndexError:
            subcommand = 'help'  # Display help if no arguments were given.

        # Preprocess options to extract --settings and --pythonpath.
        # These options could affect the commands that are available, so they
        # must be processed early.
        parser = CommandParser(
            None, usage="%(prog)s subcommand [options] [args]", add_help=False)
        parser.add_argument('args', nargs='*')  # catch-all
        with suppress(CommandError):  # Ignore any option errors at this point.
            options, args = parser.parse_known_args(self.argv[2:])

        if subcommand == 'help':
            if '--commands' in args:
                sys.stdout.write(
                    self.main_help_text(commands_only=True) + '\n')
            elif len(options.args) < 1:
                sys.stdout.write(self.main_help_text() + '\n')
            else:
                self.fetch_command(
                    options.args[0]).print_help(
                    self.prog_name, options.args[0])

        elif subcommand == 'version' or self.argv[1:] == ['--version']:
            sys.stdout.write(uvtor.VERSION + '\n')
        elif self.argv[1:] in (['--help'], ['-h']):
            sys.stdout.write(self.main_help_text() + '\n')
        else:
            self.fetch_command(subcommand).run_from_argv(self.argv)


def execute_from_command_line(argv=None):
    """Run a ManagementUtility."""
    utility = ManagementUtility(argv)
    utility.execute()



from aiocache import caches
import tornado.httpclient
import os
import sys


class Settings(object):

    def __init__(self):
        sys.path.insert(0, os.getcwd())
        settings = __import__('settings')
        for _ in dir(settings):
            if not (_.startswith('__') and _.endswith('__')):
                setattr(self, _, getattr(settings, _))
        self._set_http_max_client()
        self._set_cache()

    def _set_http_max_client(self):
        if not hasattr(self, 'HTTP_MAX_CLIENT'):
            setattr(self, 'HTTP_MAX_CLIENT', 1000)
        tornado.httpclient.AsyncHTTPClient.configure(
            None, max_clients=int(self.HTTP_MAX_CLIENT))

    def _set_cache(self):
        if not hasattr(self, 'CACHES'):
            return
        caches.set_config(self.CACHES)


try:
    settings = Settings()
except ModuleNotFoundError:
    settings = None


__ALL__ = [
    settings
]



from uvtor.core.management.base import BaseCommand, CommandError
from uvtor.db.migrations import Router
from uvtor.conf import settings
from playhouse.pool import PooledPostgresqlExtDatabase
import os


class Command(BaseCommand):
    def check(self):
        if not os.path.exists('.uvtor'):
            raise CommandError('Invalid uvtor project')

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument('-e', '--env', type=str,
                            help='Env.', default='TEST')

    def execute(self, *args, **options):
        self.check()
        _env = options.get('env')
        conf = getattr(settings, 'ENVS')[_env]

        db = PooledPostgresqlExtDatabase(
            conf.get('database'), user=conf.get('user'),
            host=conf.get('host'),
            password=conf.get('password'),
            register_hstore=False)
        db.connect()
        router = Router(db)
        router.run()
        db.close()



from io import StringIO
import nginx
import pkgutil
import configparser
import os
import getpass


def create_nginx_config(
        domain,
        listen,
        log_dir,
        start_port=4500):
    conf = nginx.Conf()
    _apps = __import__('apps')
    _port = start_port
    _server = nginx.Server()
    _server.add(
        nginx.Key('listen', listen),
        nginx.Key('server_name', domain),
        nginx.Key('client_max_body_size', '10m'),

    )
    for loader, module_name, is_pkg in pkgutil.iter_modules(_apps.__path__):
        _module = loader.find_module(module_name).load_module(module_name)
        if hasattr(_module, '__APPID__'):
            _appid = _module.__APPID__
        else:
            _appid = module_name.replace('_', '-')
        _n_workers = _module.__WORKERS__
        _upstream = []
        for _i in range(_n_workers):
            _upstream.append(nginx.Key('server', f'127.0.0.1:{_port}'))
            _port += 1
        conf.add(nginx.Upstream(
            _appid, *_upstream
            )
        )
        _server.add(
            nginx.Location(
                f'~ ^/api/{_appid}/v[0-9]/',
                nginx.Key('rewrite', f'^/api/{_appid}/(.*) /$1  break'),
                nginx.Key('proxy_pass', f'http://{_appid}'),
                nginx.Key('proxy_http_version', '1.1'),
                nginx.Key('proxy_set_header', 'Upgrade $http_upgrade'),
                nginx.Key('proxy_set_header', 'Connection "upgrade"'),
                nginx.Key('proxy_set_header', 'X-real-ip $remote_addr'),
                nginx.Key('proxy_set_header', 'X-Forwarded-For $remote_addr'),
            )
        )
    conf.add(
        nginx.Key('access_log', os.path.join(log_dir, f'apis-access.log'))
    )
    conf.add(_server)
    return nginx.dumps(conf)


def create_supervisor_config(
        log_dir,
        user=getpass.getuser(),
        directory=os.getcwd(),
        start_port=4500):
    config = configparser.SafeConfigParser()
    _programs = []
    _apps = __import__('apps')
    _port = start_port

    for loader, module_name, is_pkg in pkgutil.iter_modules(_apps.__path__):
        _module = loader.find_module(module_name).load_module(module_name)
        if hasattr(_module, '__APPID__'):
            _appid = _module.__APPID__
        else:
            _appid = module_name
        _n_workers = _module.__WORKERS__
        for _i in range(_n_workers):
            _ = {}
            _['directory'] = directory
            _['command'] = (
                f'/home/{user}/anaconda3/bin/python'
                f' -u -m uvtor runapp {module_name} --port={_port}')
            _['user'] = user
            _['autostart'] = 'true'
            _['autorestart'] = 'true'
            _['stopasgroup'] = 'true'
            _['killasgroup'] = 'true'
            _['redirect_stderr'] = 'true'
            _['stdout_logfile'] = os.path.join(
                log_dir, f'{module_name}_api.log')
            _['stdout_logfile_maxbytes'] = '200MB'
            _['stdout_logfile_backups'] = 5
            _['loglevel'] = 'INFO'
            _['environment'] = 'DEBUG="False",PYTHONIOENCODING="utf-8"'
            config[f'program:{_appid}{_i}'] = _
            _programs.append(f'{_appid}{_i}')
            _port += 1
    config['group:apis'] = {
        'programs': ','.join(_programs)
    }
    with StringIO() as _io:
        config.write(_io)
        return _io.getvalue()



from uvtor.core.management.base import BaseCommand, CommandError
import os
import shutil
import re
import uvtor


class Command(BaseCommand):

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of app',
        )

    def check(self):
        if not os.path.exists('.uvtor'):
            raise CommandError('Invalid uvtor project')

    def _is_valid_name(self, name):
        if not re.match(r'^[a-zA-Z]+[\w_]+$', name):
            raise CommandError('Invalid project name')

    def _init_handler(self, _appname):
        for dir_name, subdir_list, file_list in os.walk(
                os.path.join('apps', _appname)):
            for _file in file_list:
                if not _file.endswith('.tpl'):
                    continue
                _path = os.path.join(dir_name, _file[:-4])
                shutil.move(
                    os.path.join(dir_name, _file),
                    _path,
                    )
                if _file == 'main.py.tpl':
                    with open(_path) as _f:
                        _handler = _f.read()
                    _handler = _handler.format(
                        appname=_appname
                        )
                    with open(_path, 'w') as _f:
                        _f.write(_handler)

    def _init_test(self, _appname):
        _appname_upper = ''.join(
            [_p.capitalize() for _p in _appname.split('_')])
        for dir_name, subdir_list, file_list in os.walk(
                os.path.join('tests', _appname)):
            for _file in file_list:
                if not _file.endswith('.tpl'):
                    continue
                _path = os.path.join(dir_name, _file[:-4])
                shutil.move(
                    os.path.join(dir_name, _file),
                    _path,
                    )
                if _file == 'v0.py.tpl':
                    with open(_path) as _f:
                        _test = _f.read()
                    _test = _test.format(
                        appname_upper=_appname_upper
                        )
                    with open(_path, 'w') as _f:
                        _f.write(_test)

    def _init_routes(self, _appname):
        _path = os.path.join(
            'apps',
            _appname, 'routes', 'v0', '__init__.py')
        for dir_name, subdir_list, file_list in os.walk(
                os.path.join('apps', _appname, 'routes')):
            for _file in file_list:
                if not _file.endswith('.tpl'):
                    continue
                _path = os.path.join(dir_name, _file[:-4])
                shutil.move(
                    os.path.join(dir_name, _file),
                    _path,
                    )
        with open(_path) as _f:
            _routes = _f.read()
            _routes = _routes.format(
                appname=_appname
                )
        with open(_path, 'w') as _f:
            _f.write(_routes)

    def execute(self, **options):
        _appname = options.get('name')
        self._is_valid_name(_appname)
        _template_dir = os.path.join(
            uvtor.__path__[0], '_templates', 'app_template')
        _test_template_dir = os.path.join(
            uvtor.__path__[0], '_templates', 'test_template')

        shutil.copytree(_template_dir, os.path.join('apps', _appname))
        shutil.copytree(_test_template_dir, os.path.join('tests', _appname))

        self._init_handler(_appname)
        self._init_test(_appname)
        self._init_routes(_appname)

        print(f'Successfully create app {_appname}')



from tornado.testing import AsyncHTTPTestCase
from tornado.platform.asyncio import AsyncIOMainLoop
from os.path import dirname
from uvtor.conf import settings
from uvtor.core import make_app

import sys
import os


class BaseTestCase(AsyncHTTPTestCase):

    def get_app(self):
        name = dirname(sys.modules[self.__module__].__file__).split('/')[-1]
        try:
            sys.path.index(os.path.join(settings.ROOT_DIR, 'apps', name))
        except ValueError:
            sys.path.insert(1, os.path.join(settings.ROOT_DIR, 'apps', name))
        return make_app(name, False)

    def get_new_ioloop(self):
        return AsyncIOMainLoop()



from uvtor.core import response as _resp


class APIException(Exception):

    code = _resp.CODE_1_ERROR
    default_detail = 'A server error occurred.'
    default_detail_zh = '服务器发生了一个错误'

    def __init__(self, detail=None, chinese_detail=True):
        if detail is not None:
            self.detail = detail
        else:
            self.detail = self.default_detail
            if chinese_detail:
                self.detail = self.default_detail_zh

    def __str__(self):
        return self.detail


class LoginRequired(APIException):
    code = _resp.CODE_2_LOGIN_REQUIRED
    default_detail = 'Login required.'

    default_detail_zh = '请登陆后再进行此操作'


class SecureTokenError(APIException):

    code = _resp.CODE_3_ERROR_SECURE_TOKEN
    default_detail = 'Error secure token.'


class UsernameOrPasswordError(APIException):

    code = _resp.CODE_4_ERROR_USERNAME_OR_PASSWORD
    default_detail = 'Error username or password.'

    default_detail_zh = '错误的用户名或密码'


class InvalidPassword(APIException):

    code = _resp.CODE_5_INVALID_PASSWORD
    default_detail = 'Invalid password'

    default_detail_zh = '密码不符合要求'


class UserCreationError(APIException):

    code = _resp.CODE_6_USER_CREATION_FAILED
    default_detail = 'UserCreationError.'

    default_detail_zh = '创建用户失败'


class SMSSendFailed(APIException):

    code = _resp.CODE_7_SEND_SMS_FAILED
    default_detail = 'Send SMS failed'

    default_detail_zh = '短信发送失败'


class CaptchaValidationError(APIException):

    code = _resp.CODE_8_CAPTCHA_VALIDATION_ERROR
    default_detail = 'Captcha validation error'

    default_detail_zh = '验证码不匹配'


class InvalidPhoneNumber(APIException):

    code = _resp.CODE_9_INVALID_PHONE_NUMBER
    default_detail = 'Invalid phone number'

    default_detail_zh = '不正确的手机号码'


class InvalidEmailAddress(APIException):

    code = _resp.CODE_10_INVALID_EMAIL_ADDRESS
    default_detail = 'Invalid email address'

    default_detail_zh = '不正确的邮件地址'


class InvalidUsername(APIException):

    code = _resp.CODE_11_INVALID_USERNAME
    default_detail = 'Invalid username'

    default_detail_zh = '不合法的用户名'


class InvalidChineseName(APIException):

    code = _resp.CODE_12_INVALID_CHINESE_NAME
    default_detail = 'Invalid Chinese name'

    default_detail_zh = '不正确的中文姓名'


class VcodeValidationError(APIException):

    code = _resp.CODE_13_VCODE_VALIDATION_ERROR
    default_detail = 'Vcode validation error'

    default_detail_zh = '验证码不正确或已经失效'


class PhoneAlreadyExistsError(APIException):

    code = _resp.CODE_14_PHONE_ALREADY_EXISTS_ERROR
    default_detail = 'Phone already exists'

    default_detail_zh = '手机号已经被注册'


class UsernameAlreadyExistsError(APIException):

    code = _resp.CODE_15_USERNAME_ALREADY_EXISTS_ERROR
    default_detail = 'Username already exists'

    default_detail_zh = '用户名已存在'


class EmailAlreadyExistsError(APIException):

    code = _resp.CODE_16_EMAIL_ALREADY_EXISTS_ERROR
    default_detail = 'Email already exists'

    default_detail_zh = '邮箱地址已经被注册'


class MissingRequiredParams(APIException):

    code = _resp.CODE_19_MISSING_REQUIRED_PARAMS
    default_detail = 'Required params is missing'

    default_detail_zh = '缺少必要参数'


class PermissionDenied(APIException):

    code = _resp.CODE_20_PERMISSION_DENIED
    default_detail = 'Permission denied'

    default_detail_zh = '没有权限查看此内容'


class InvalidParameterValue(APIException):

    code = _resp.CODE_21_INVALID_PARAMETER
    default_detail = 'Invalid parameter value'

    default_detail_zh = '不正确的参数值'


class ObjectAlreadyExists(APIException):

    code = _resp.CODE_23_OBJECT_ALREADY_EXISTS
    default_detail = 'Object already exists'

    default_detail_zh = '此项已经存在'


class UserIsDisabled(APIException):

    code = _resp.CODE_24_USER_IS_DISABLED
    default_detail = 'The user is disabled'

    default_detail_zh = '该用户已被禁用'



from uvtor.conf import settings
from playhouse import postgres_ext
from playhouse import pool
import peewee_async
import datetime


database = peewee_async.PooledPostgresqlDatabase(
    settings.DB_DATABASE, user=settings.DB_USER,
    host=settings.DB_HOST,
    password=settings.DB_PASSWORD,
    min_connections=settings.DB_CONN_POOL_MIN_SIZE,
    max_connections=settings.DB_CONN_POOL_MAX_SIZE)

pg_db = pool.PooledPostgresqlExtDatabase(
    settings.DB_DATABASE, user=settings.DB_USER,
    host=settings.DB_HOST,
    password=settings.DB_PASSWORD,
    min_connections=settings.DB_CONN_POOL_MIN_SIZE,
    max_connections=settings.DB_CONN_POOL_MAX_SIZE
    )

manager = peewee_async.Manager(database)


class BaseModel(postgres_ext.Model):

    created = postgres_ext.DateTimeTZField(
        verbose_name='添加时间',
        default=datetime.datetime.now,
        null=False)
    modified = postgres_ext.DateTimeTZField(
        verbose_name='更新时间',
        default=datetime.datetime.now,
        null=False)

    def save(self, *args, **kwargs):
        self.modified = datetime.datetime.now()
        return super(BaseModel, self).save(*args, **kwargs)

    class Meta:
        database = database



from uvtor.core.management.base import BaseCommand, CommandError
import os
import shutil
import re
import uvtor
import base64
import uuid


class Command(BaseCommand):

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of project',
        )

    def _is_valid_name(self, name):
        if not re.match(r'^[a-zA-Z]+[\w_]+$', name):
            raise CommandError('Invalid project name')

    def _init_settings(self, path):
        with open(path) as _f:
            _settings = _f.read()
        _settings = _settings.format(
            cookie_secret="'{}'".format(base64.b64encode(
                uuid.uuid4().bytes + uuid.uuid4().bytes).decode())
            )

        with open(path, 'w') as _f:
            _f.write(_settings)

    def execute(self, **options):
        _projname = options.get('name')
        self._is_valid_name(_projname)
        _template_dir = os.path.join(
            uvtor.__path__[0], '_templates', 'project_template')

        shutil.copytree(_template_dir, _projname)

        for dir_name, subdir_list, file_list in os.walk(_projname):
            for _file in file_list:
                if not _file.endswith('.tpl'):
                    continue
                shutil.move(
                    os.path.join(dir_name, _file),
                    os.path.join(dir_name, _file[:-4]),
                    )
                if _file == 'settings.py.tpl':
                    self._init_settings(
                        os.path.join(dir_name, _file[:-4]))
        print(f'Successfully create project {_projname}')



from uvtor.core.management.base import BaseCommand
from uvtor.utils import config
import os
import sys
import getpass


class Command(BaseCommand):

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'name', metavar='name',
            help='Name of software',
        )
        parser.add_argument(
            '--domain', '-D', metavar='domain',
            help='Domain',
        )
        parser.add_argument(
            '--listen', '-P', metavar='listen',
            default=80,
            help='listening port',
        )
        parser.add_argument(
            '--log_dir', '-L', metavar='log_dir',
            help='Log directory',
        )
        parser.add_argument(
            '--user', '-U', metavar='user',
            default=getpass.getuser(),
            help='User',
        )

    def execute(self, **options):
        sys.path.insert(0, os.getcwd())
        _name = options.get('name')
        if _name == 'nginx':
            print(config.create_nginx_config(
                options.get('domain'),
                options.get('listen'),
                options.get('log_dir')
                ))
        elif _name == 'supervisor':
            print(config.create_supervisor_config(
                options.get('log_dir'),
                options.get('user'),
                ))
